package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.pdf.PdfReader;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.dto.PdfDto;
import com.qmth.distributed.print.business.config.DictionaryConfig;
import com.qmth.distributed.print.business.entity.BasicAttachment;
import com.qmth.distributed.print.business.entity.ExamDetail;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.enums.ClassifyEnum;
import com.qmth.distributed.print.business.enums.PageSizeEnum;
import com.qmth.distributed.print.business.enums.UploadFileEnum;
import com.qmth.distributed.print.business.mapper.BasicAttachmentMapper;
import com.qmth.distributed.print.business.service.BasicAttachmentService;
import com.qmth.distributed.print.business.util.HtmlToPdfUtil;
import com.qmth.distributed.print.business.util.OssUtil;
import com.qmth.distributed.print.business.util.PdfUtil;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.HexUtils;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * <p>
 * 所有附件记录表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicAttachmentServiceImpl extends ServiceImpl<BasicAttachmentMapper, BasicAttachment> implements BasicAttachmentService {
    private final static Logger log = LoggerFactory.getLogger(BasicAttachmentServiceImpl.class);

    @Resource
    OssUtil ossUtil;

    @Autowired
    private DictionaryConfig dictionaryConfig;

    /**
     * 保存附件
     *
     * @param file
     * @param md5
     * @param type
     * @return
     */
    @Override
    @Transactional
    public BasicAttachment saveAttachment(MultipartFile file, String md5, UploadFileEnum type) {
        return saveAttachmentCommon(file, md5, type, null);
    }

    /**
     * 保存附件
     *
     * @param file
     * @param md5
     * @param type
     * @param objId
     * @return
     */
    @Override
    @Transactional
    public BasicAttachment saveAttachment(MultipartFile file, String md5, UploadFileEnum type, Long objId) {
        return saveAttachmentCommon(file, md5, type, objId);
    }

    /**
     * 保存html附件
     *
     * @param fileName
     * @param htmlContent
     * @param userId
     * @param localFileList
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public BasicAttachment saveAttachmentHtml(String fileName, String htmlContent, Long userId, List<PdfDto> localFileList) throws IOException {
        BasicAttachment basicAttachment = null;
        PdfDto pdfDto = null;
        try {
            byte[] bytes = htmlContent.getBytes();
            int size = bytes.length;
            boolean oss = dictionaryConfig.sysDomain().isOss();
            LocalDateTime nowTime = LocalDateTime.now();
            StringJoiner stringJoiner = new StringJoiner("");
            if (!oss) {
                stringJoiner.add(SystemConstant.TEMP_FILES_DIR).add(File.separator);
            }
            stringJoiner.add(UploadFileEnum.HTML.getTitle()).add(File.separator)
                    .add(String.valueOf(nowTime.getYear())).add(File.separator)
                    .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                    .add(String.format("%02d", nowTime.getDayOfMonth()));

            JSONObject jsonObject = new JSONObject();
            stringJoiner.add(File.separator).add(SystemConstant.getUuid()).add(SystemConstant.HTML_PREFIX);
            String fileMd5 = null;
            if (oss) {//上传至oss
                String dirName = stringJoiner.toString().replaceAll("\\\\", "/");
                ossUtil.ossUpload(dirName, htmlContent);
                jsonObject.put(SystemConstant.TYPE, SystemConstant.OSS);
                jsonObject.put(SystemConstant.PATH, dirName);
                String url = SystemConstant.TEMP_FILES_DIR + File.separator + dirName;
                File localHtmlFile = ossUtil.ossDownload(dirName, url);
                StringJoiner pdfStringJoiner = new StringJoiner("");
                pdfStringJoiner.add(UploadFileEnum.PDF.getTitle()).add(File.separator);
                pdfStringJoiner.add(String.valueOf(nowTime.getYear())).add(File.separator)
                        .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                        .add(String.format("%02d", nowTime.getDayOfMonth()));
                pdfStringJoiner.add(File.separator).add(SystemConstant.getUuid()).add(SystemConstant.PDF_PREFIX);

                String pdfDirName = pdfStringJoiner.toString();
                String destUrl = SystemConstant.PDF_TEMP_FILES_DIR + File.separator + pdfDirName;
//                destUrl = destUrl.replaceAll("\\\\","/");
                HtmlToPdfUtil.convert(localHtmlFile.getPath(), destUrl, PageSizeEnum.A3);
                File pdfFile = new File(destUrl);
                if (!pdfFile.exists()) {
                    pdfFile.getParentFile().mkdirs();
                    pdfFile.createNewFile();
                }

                pdfDto = PdfUtil.addPdfPage(pdfFile);
                localFileList.add(new PdfDto(pdfFile.getPath(), PageSizeEnum.A3, pdfDto.getPageCount()));
                fileMd5 = DigestUtils.md5Hex(new FileInputStream(pdfFile));
                ossUtil.ossUpload(pdfDirName, pdfFile, BinaryUtil.toBase64String(HexUtils.decodeHex(fileMd5)));
//                localHtmlFile.delete();
                jsonObject.put(SystemConstant.PDF_PATH, pdfDirName);
            } else {//上传至服务器
                File finalFile = new File(stringJoiner.toString());
                if (!finalFile.exists()) {
                    finalFile.getParentFile().mkdirs();
                    finalFile.createNewFile();
                }
                FileCopyUtils.copy(bytes, new File(stringJoiner.toString()));
                fileMd5 = DigestUtils.md5Hex(new FileInputStream(stringJoiner.toString()));
                jsonObject.put(SystemConstant.TYPE, SystemConstant.LOCAL);
                jsonObject.put(SystemConstant.PATH, stringJoiner.toString());
                String destUrl = finalFile.getPath().replaceAll(SystemConstant.HTML_PREFIX, SystemConstant.PDF_PREFIX).replaceAll(UploadFileEnum.HTML.name().toLowerCase(), UploadFileEnum.PDF.name().toLowerCase());
                HtmlToPdfUtil.convert(finalFile.getPath(), destUrl, PageSizeEnum.A3);
                jsonObject.put(SystemConstant.PDF_PATH, destUrl);
            }
            jsonObject.put(SystemConstant.UPLOAD_TYPE, new UploadFileEnum[]{
                    UploadFileEnum.HTML, UploadFileEnum.PDF
            });
            basicAttachment = new BasicAttachment(jsonObject.toJSONString(), fileName, SystemConstant.HTML_PREFIX, new BigDecimal(size), fileMd5, userId);
            basicAttachment.setPages(pdfDto.getActualPageCount());
            save(basicAttachment);
        } catch (Exception e) {
            log.error("请求出错", e);
            deleteAttachment(basicAttachment);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return basicAttachment;
    }

    /**
     * 保存html附件
     *
     * @param fileName
     * @param htmlContent
     * @return
     * @throws IOException
     */
    @Override
    public File saveAttachmentHtml(String rootPath, String fileName, String htmlContent) {
        File file = null;
        try {
            byte[] bytes = htmlContent.getBytes();
            //上传至服务器
            File mkdir = new File(rootPath);
            if (!mkdir.exists()) {
                mkdir.mkdirs();
            }
            file = new File(rootPath, fileName);
            FileCopyUtils.copy(bytes, file);
            //计算html文件md5
        } catch (Exception e) {
            e.printStackTrace();
        }
        return file;
    }

    /**
     * 保存附件
     *
     * @param dirName
     * @param userId
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public BasicAttachment saveAttachmentPdf(String dirName, Long userId) throws IOException {
        BasicAttachment basicAttachment = null;
        try {
            StringJoiner stringJoiner = new StringJoiner("").add(SystemConstant.PDF_TEMP_FILES_DIR).add(File.separator);
            File localPdfFile = new File(stringJoiner.toString() + File.separator + dirName);
            if (!localPdfFile.getParentFile().exists()) {
                localPdfFile.getParentFile().mkdirs();
                localPdfFile.createNewFile();
            }
            JSONObject jsonPdf = new JSONObject();
            jsonPdf.put(SystemConstant.TYPE, SystemConstant.OSS);
            jsonPdf.put(SystemConstant.PATH, dirName);
            jsonPdf.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.PDF);
            basicAttachment = new BasicAttachment(jsonPdf.toJSONString(), localPdfFile.getName(), SystemConstant.PDF_PREFIX, new BigDecimal(localPdfFile.length()), DigestUtils.md5Hex(new FileInputStream(localPdfFile)), userId);
            save(basicAttachment);
        } catch (Exception e) {
            log.error("请求出错", e);
            deleteAttachment(basicAttachment);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return basicAttachment;
    }

    /**
     * 保存附件
     *
     * @param classifyEnum
     * @param examDetail
     * @param basicAttachment
     * @param pdfList
     * @param printCount
     * @param sequence
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public BasicAttachment saveAttachmentPdf(ClassifyEnum classifyEnum, ExamDetail examDetail, BasicAttachment basicAttachment, List<PdfDto> pdfList, Integer printCount, Integer sequence) throws IOException, DocumentException {
        try {
            boolean oss = dictionaryConfig.sysDomain().isOss();
            JSONObject jsonObject = JSONObject.parseObject(basicAttachment.getPath());
            String type = basicAttachment.getType();
            String filePath = null;
            String url = null;
            File htmlFile = null;
            if (Objects.nonNull(type) && Objects.equals(type, SystemConstant.FTL_PREFIX)) {
                filePath = (String) jsonObject.get(SystemConstant.HTML_PATH);
                url = SystemConstant.TEMP_FILES_DIR + File.separator + filePath;
                htmlFile = new File(url);
            } else if (Objects.nonNull(type) && Objects.equals(type, SystemConstant.HTML_PREFIX)) {
                filePath = (String) jsonObject.get(SystemConstant.PATH);
                url = SystemConstant.TEMP_FILES_DIR + File.separator + filePath;
                htmlFile = ossUtil.ossDownload(filePath, url);
            }
            LocalDateTime nowTime = LocalDateTime.now();
            StringJoiner pdfStringJoiner = new StringJoiner("");
            if (!oss) {
                pdfStringJoiner.add(SystemConstant.TEMP_FILES_DIR).add(File.separator);
            }
            pdfStringJoiner.add(UploadFileEnum.PDF.getTitle()).add(File.separator);
            pdfStringJoiner.add(String.valueOf(nowTime.getYear())).add(File.separator)
                    .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                    .add(String.format("%02d", nowTime.getDayOfMonth()));
            pdfStringJoiner.add(File.separator).add(SystemConstant.getUuid()).add(SystemConstant.PDF_PREFIX);

            String pdfDirName = pdfStringJoiner.toString();
            String destUrl = SystemConstant.PDF_TEMP_FILES_DIR + File.separator + pdfDirName;
            File pdfFile = new File(destUrl);
            if (!pdfFile.exists()) {
                pdfFile.getParentFile().mkdirs();
                pdfFile.createNewFile();
            }

            HtmlToPdfUtil.convert(htmlFile.getPath(), destUrl, PageSizeEnum.A4);
            String htmlFileMd5 = DigestUtils.md5Hex(new FileInputStream(htmlFile));
            String pdfFileMd5 = DigestUtils.md5Hex(new FileInputStream(pdfFile));
            PdfDto pdfDto = null;
            if (oss) {//上传至oss
                pdfDto = PdfUtil.addPdfPage(pdfFile);
                ossUtil.ossUpload(pdfDirName, pdfFile, BinaryUtil.toBase64String(HexUtils.decodeHex(pdfFileMd5)));
//                htmlFile.delete();
            }
            JSONObject attachmentPath = JSONObject.parseObject(examDetail.getAttachmentPath());
            attachmentPath = Objects.isNull(attachmentPath) ? new JSONObject() : attachmentPath;
            JSONArray jsonArray = (JSONArray) attachmentPath.get(SystemConstant.PATH);
            jsonArray = Objects.isNull(jsonArray) ? new JSONArray() : jsonArray;
            JSONObject object = new JSONObject();
            object.put("printType", classifyEnum.name());
            object.put(SystemConstant.HTML_PATH, filePath);
            object.put("htmlMd5", htmlFileMd5);
            object.put("pdfMd5", pdfFileMd5);
            object.put(SystemConstant.PDF_PATH, pdfDirName);
            object.put(SystemConstant.TYPE, SystemConstant.OSS);
            object.put(SystemConstant.UPLOAD_TYPE, new UploadFileEnum[]{
                    UploadFileEnum.HTML,
                    UploadFileEnum.PDF
            });
            jsonArray.add(object);
            attachmentPath.put(SystemConstant.PATH, jsonArray);
            examDetail.setAttachmentPath(attachmentPath.toJSONString());
            basicAttachment.setPages(pdfDto.getActualPageCount());
            for (int i = 0; i < printCount; i++) {
                pdfList.add(new PdfDto(pdfFile.getPath(), PageSizeEnum.A4, pdfDto.getPageCount(), sequence));
            }
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return basicAttachment;
    }

    /**
     * 删除附件
     *
     * @param basicAttachment
     */
    @Override
    public void deleteAttachment(BasicAttachment basicAttachment) {
        deleteAttachmentCommon(basicAttachment);
    }

    /**
     * 批量删除附件
     *
     * @param basicAttachmentList
     */
    @Override
    public void batchDeleteAttachment(List<BasicAttachment> basicAttachmentList) {
        if (Objects.nonNull(basicAttachmentList) && basicAttachmentList.size() > 0) {
            for (BasicAttachment basicAttachment : basicAttachmentList) {
                deleteAttachmentCommon(basicAttachment);
            }
        }
    }

    /**
     * 删除附件公用
     *
     * @param basicAttachment
     */
    public void deleteAttachmentCommon(BasicAttachment basicAttachment) {
        if (Objects.nonNull(basicAttachment) && Objects.nonNull(basicAttachment.getPath())) {
            JSONObject jsonObject = JSONObject.parseObject(basicAttachment.getPath());
            String type = String.valueOf(jsonObject.get(SystemConstant.TYPE));
            if (Objects.nonNull(type) && Objects.equals(type, SystemConstant.OSS)) {//删除阿里云附件
                ossUtil.ossDelete(jsonObject.get(SystemConstant.PATH).toString());
                if (Objects.nonNull(jsonObject.get(SystemConstant.PDF_PATH))) {
                    ossUtil.ossDelete(jsonObject.get(SystemConstant.PDF_PATH).toString());
                }
            } else {//删除服务器附件
                File file = new File(jsonObject.get(SystemConstant.PATH).toString());
                file.delete();
                if (Objects.nonNull(jsonObject.get(SystemConstant.PDF_PATH))) {
                    file = new File(jsonObject.get(SystemConstant.PDF_PATH).toString());
                    file.delete();
                }
            }
        }
    }

    /**
     * 保存附件公用
     *
     * @param file
     * @param md5
     * @param type
     * @param objId
     * @return
     */
    public BasicAttachment saveAttachmentCommon(MultipartFile file, String md5, UploadFileEnum type, Long objId) {
        if (Objects.isNull(type)) {
            throw ExceptionResultEnum.ATTACHMENT_TYPE_EMPTY.exception();
        }
        BasicAttachment basicAttachment = null;
        try {
            SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
            int temp = file.getOriginalFilename().indexOf(".");
            String fileName = file.getOriginalFilename().substring(0, temp);
            String format = file.getOriginalFilename().substring(temp, file.getOriginalFilename().length());
            List<String> attachmentTypeList = dictionaryConfig.sysDomain().getAttachmentType();
            if (Objects.nonNull(format)) {
                long count = attachmentTypeList.stream().filter(s -> format.equalsIgnoreCase(s)).count();
                if (count == 0) {
                    throw ExceptionResultEnum.ERROR.exception("文件格式只能为" + attachmentTypeList.toString());
                }
            }
            int attachmentLength = dictionaryConfig.sysDomain().getAttachmentLength().intValue();
            if (Objects.nonNull(fileName) && fileName.length() > attachmentLength) {
                throw ExceptionResultEnum.ERROR.exception("文件名长度不能超过" + attachmentLength + "个字符");
            }
            long size = file.getSize();
            BigDecimal b = new BigDecimal(size);
            BigDecimal num = new BigDecimal(1024);
            b = b.divide(num, 2, BigDecimal.ROUND_HALF_UP).divide(num, 2, BigDecimal.ROUND_HALF_UP)
                    .setScale(2, BigDecimal.ROUND_HALF_UP);
            double attachmentSize = dictionaryConfig.sysDomain().getAttachmentSize().doubleValue();
            if (b.doubleValue() > attachmentSize) {
                throw ExceptionResultEnum.ERROR.exception("文件大小不能超过" + attachmentSize + "MB");
            }
            log.info("fileName:{}", fileName);
            log.info("format:{}", format);
            log.info("size:{}", b);
            log.info("getOriginalFilename:{}", file.getOriginalFilename());
            String fileMd5 = DigestUtils.md5Hex(file.getBytes());
            log.info("fileMd5:{}", fileMd5);
            log.info("md5:{}", md5);
            if (!Objects.equals(fileMd5, md5)) {
                throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
            }
            boolean oss = dictionaryConfig.sysDomain().isOss();
            LocalDateTime nowTime = LocalDateTime.now();
            StringJoiner stringJoiner = new StringJoiner("");
            if (!oss) {
                stringJoiner.add(SystemConstant.TEMP_FILES_DIR).add(File.separator);
            }
            stringJoiner.add(type.getTitle()).add(File.separator);
//            else if (type == UploadFileEnum.PAPER) {//试卷需要单独
//                stringJoiner.add(type.getTitle()).add(File.separator);
//            } else if (type == UploadFileEnum.UPLOAD) {
//                stringJoiner.add(type.getTitle()).add(File.separator);
//            }
            stringJoiner.add(String.valueOf(nowTime.getYear())).add(File.separator)
                    .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                    .add(String.format("%02d", nowTime.getDayOfMonth()));

            JSONObject jsonObject = new JSONObject();
            stringJoiner.add(File.separator).add(SystemConstant.getUuid()).add(format);
            if (oss) {//上传至oss\
                String dirName = stringJoiner.toString().replaceAll("\\\\", "/");
                ossUtil.ossUpload(dirName, file.getInputStream(), BinaryUtil.toBase64String(HexUtils.decodeHex(md5)));
                jsonObject.put(SystemConstant.TYPE, SystemConstant.OSS);
                jsonObject.put(SystemConstant.PATH, dirName);
            } else {//上传至服务器
                File finalFile = new File(stringJoiner.toString());
                if (!finalFile.exists()) {
                    finalFile.getParentFile().mkdirs();
                    finalFile.createNewFile();
                }
                FileUtils.copyInputStreamToFile(file.getInputStream(), finalFile);
                jsonObject.put(SystemConstant.TYPE, SystemConstant.LOCAL);
                jsonObject.put(SystemConstant.PATH, stringJoiner.toString());
            }
            jsonObject.put(SystemConstant.UPLOAD_TYPE, type);

            basicAttachment = new BasicAttachment(jsonObject.toJSONString(), fileName, format, b, fileMd5, requestUser.getId(), objId);
            save(basicAttachment);

            // pdf需要读取总页数
            Integer pages = 0;
            if (".pdf".equals(format)) {
                PdfReader pdfReader = new PdfReader(file.getInputStream());
                pages = pdfReader.getNumberOfPages();
            }
            basicAttachment.setPages(pages);
        } catch (Exception e) {
            log.error("请求出错", e);
            deleteAttachment(basicAttachment);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return basicAttachment;
    }
}
