package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.entity.BasicCourse;
import com.qmth.distributed.print.business.entity.ExamDetailCourse;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.mapper.ExamDetailCourseMapper;
import com.qmth.distributed.print.business.service.ExamDetailCourseService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 考务数据（考场关联科目） 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamDetailCourseServiceImpl extends ServiceImpl<ExamDetailCourseMapper, ExamDetailCourse> implements ExamDetailCourseService {

    @Transactional(rollbackFor = Exception.class)
    @Override
    public double calculatePackagesByDetailId(Long examDetailId) {
        System.out.println(this.count(new QueryWrapper<ExamDetailCourse>().lambda().eq(ExamDetailCourse::getExamDetailId,examDetailId)));
        return this.count(new QueryWrapper<ExamDetailCourse>().lambda().eq(ExamDetailCourse::getExamDetailId,examDetailId));
    }

    @Override
    public List<Map<String, Object>> listByExamDetailId(Long examDetailId) {
        return this.baseMapper.listByExamDetailId(examDetailId, ExamDetailStatusEnum.FINISH.name());
    }

    @Override
    public List<ExamDetailCourse> listDetailCourseByCourseCodeAndPaperNumber(Long schoolId, String courseCode, String paperNumber) {
        QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
        examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getSchoolId, schoolId)
                .eq(ExamDetailCourse::getCourseCode, courseCode)
                .eq(ExamDetailCourse::getPaperNumber, paperNumber);
        List<ExamDetailCourse> examDetailCourses = this.list(examDetailCourseQueryWrapper);
        return examDetailCourses;
    }

    @Override
    public void updatePaperNumber(List<ExamDetailCourse> examDetailCourses, String paperNumber, String relatePaperType) {
        List<Long> examDetailCourseIds = examDetailCourses.stream().map(m->m.getId()).collect(Collectors.toList());
        UpdateWrapper<ExamDetailCourse> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetailCourse::getPaperNumber, paperNumber)
                .set(ExamDetailCourse::getPaperType, relatePaperType)
                .in(ExamDetailCourse::getId, examDetailCourseIds);
        this.update(updateWrapper);
    }

    @Override
    public List<ExamDetailCourse> listByExamDetailIdAndStatus(Long examDetailId) {
        return this.baseMapper.listByExamDetailIdAndStatus(examDetailId, ExamDetailStatusEnum.FINISH.name());
    }

    @Override
    public List<BasicCourse> listCoursesByPrintPlanId(String param, Long printPlanId) {
        return this.baseMapper.listCoursesByPrintPlanId(param, printPlanId);
    }

    @Override
    public List<String> listPaperNumberByPrintPlanId(String param, Long printPlanId) {
        return this.baseMapper.listPaperNumberByPrintPlanId(param, printPlanId);
    }
}
