package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.OrgDto;
import com.qmth.distributed.print.business.entity.BasicTemplateOrg;
import com.qmth.distributed.print.business.entity.SysOrg;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.mapper.SysOrgMapper;
import com.qmth.distributed.print.business.service.BasicTemplateOrgService;
import com.qmth.distributed.print.business.service.SysOrgService;
import com.qmth.distributed.print.business.service.SysUserService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.ImportAutoConfiguration;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 学校组织架构 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class SysOrgServiceImpl extends ServiceImpl<SysOrgMapper, SysOrg> implements SysOrgService {

    @Autowired
    private BasicTemplateOrgService basicTemplateOrgService;

    @Autowired
    private SysUserService sysUserService;

    @Override
    public List<OrgDto> listOrgTree() {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<SysOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrg::getSchoolId, schoolId).orderByAsc(SysOrg::getParentId, SysOrg::getCode);
        List<SysOrg> orgList = this.list(queryWrapper);
        Map<Long, OrgDto> map = new LinkedHashMap<>();
        for (SysOrg sysOrg : orgList) {
            OrgDto orgDto = new OrgDto();
            orgDto.setId(String.valueOf(sysOrg.getId()));
            orgDto.setSchoolId(String.valueOf(sysOrg.getSchoolId()));
            orgDto.setCode(sysOrg.getCode());
            orgDto.setName(sysOrg.getName());
            orgDto.setParentId(sysOrg.getParentId());
            orgDto.setEnable(sysOrg.getEnable());
            map.put(sysOrg.getId(), orgDto);
        }

        Iterator<Long> iterator = map.keySet().iterator();
        Set<Long> deleteKeys = new HashSet<>();
        while (iterator.hasNext()) {
            Long parentId = iterator.next();
            if (map.get(parentId).getParentId() != null) {
                map.get(map.get(parentId).getParentId()).getChildren().add(map.get(parentId));
                deleteKeys.add(parentId);
            }
        }
        for (Long key : deleteKeys) {
            map.remove(key);
        }

        return new ArrayList<>(map.values());
    }

    @Override
    public boolean saveOrg(SysOrg org) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        org.setSchoolId(schoolId);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        QueryWrapper<SysOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrg::getSchoolId, org.getSchoolId()).eq(SysOrg::getParentId, org.getParentId()).eq(SysOrg::getCode, org.getCode());
        SysOrg sysOrg = this.getOne(queryWrapper);

        // 新增
        if (org.getId() == null) {
            if (sysOrg != null) {
                throw ExceptionResultEnum.ERROR.exception("机构代码已存在");
            }
            org.setId(SystemConstant.getDbUuid());
            org.setCreateId(sysUser.getId());
            org.setCreateTime(System.currentTimeMillis());
            org.setCreateTime(System.currentTimeMillis());
            return this.save(org);
        }
        // 修改
        else {
            if (sysOrg != null && org.getId().longValue() != sysOrg.getId().longValue()) {
                throw ExceptionResultEnum.ERROR.exception("机构代码已存在");
            }
            org.setUpdateId(sysUser.getId());
            org.setUpdateTime(System.currentTimeMillis());
            return this.updateById(org);
        }

    }

    @Override
    public boolean enable(SysOrg org) {
        // 禁用时，校验机构
        if(!org.getEnable()){
            validateOrg(org.getId());
        }
        UpdateWrapper<SysOrg> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(SysOrg::getEnable, org.getEnable()).eq(SysOrg::getId, org.getId());
        return this.update(updateWrapper);
    }

    @Override
    public boolean remove(Long id) {
        // 机构校验
        validateOrg(id);
        return this.removeById(id);
    }


    private void validateOrg(Long id){
        // 机构下是否有用户
        List<SysUser> sysUsers = sysUserService.listByOrgId(id);
        if(sysUsers != null && sysUsers.size() > 0){
            throw ExceptionResultEnum.ERROR.exception("该机构下有用户，不能删除");
        }

        // 机构下是否有子机构
        QueryWrapper<SysOrg> orgQueryWrapper = new QueryWrapper<>();
        orgQueryWrapper.lambda().eq(SysOrg::getParentId, id);
        List<SysOrg> sysOrgs = this.list(orgQueryWrapper);
        if(sysOrgs != null && sysOrgs.size() > 0){
            throw ExceptionResultEnum.ERROR.exception("该机构下有子机构，不能删除");
        }

        QueryWrapper<BasicTemplateOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicTemplateOrg::getOrgId, id);
        List<BasicTemplateOrg> basicTemplateOrgs = basicTemplateOrgService.list(queryWrapper);
        if (basicTemplateOrgs != null && basicTemplateOrgs.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("机构已绑定，不能删除");
        }
    }
}
