package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.PrivilegeDto;
import com.qmth.distributed.print.business.entity.SysPrivilege;
import com.qmth.distributed.print.business.entity.SysRolePrivilege;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.enums.PrivilegePropertyEnum;
import com.qmth.distributed.print.business.mapper.SysPrivilegeMapper;
import com.qmth.distributed.print.business.service.SysPrivilegeService;
import com.qmth.distributed.print.business.service.SysRolePrivilegeService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 菜单权限表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class SysPrivilegeServiceImpl extends ServiceImpl<SysPrivilegeMapper, SysPrivilege> implements SysPrivilegeService {

    @Autowired
    private SysRolePrivilegeService sysRolePrivilegeService;

    @Override
    public List<PrivilegeDto> listPrivilegeTree() {
        QueryWrapper<SysPrivilege> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().orderByAsc(SysPrivilege::getParentId, SysPrivilege::getSequence);
        List<SysPrivilege> orgList = this.list(queryWrapper);
        Map<Long, PrivilegeDto> map = new LinkedHashMap<>();
        for (SysPrivilege privilege : orgList) {
            PrivilegeDto privilegeDto = new PrivilegeDto();
            privilegeDto.setId(String.valueOf(privilege.getId()));
            privilegeDto.setName(privilege.getName());
            privilegeDto.setUrl(privilege.getUrl());
            privilegeDto.setType(privilege.getType());
            privilegeDto.setParentId(privilege.getParentId());
            privilegeDto.setSequence(privilege.getSequence());
            map.put(privilege.getId(), privilegeDto);
        }

        Iterator<Long> iterator = map.keySet().iterator();
        Set<Long> deleteKeys = new HashSet<>();
        while (iterator.hasNext()) {
            Long parentId = iterator.next();
            if (map.get(parentId).getParentId() != null) {
                map.get(map.get(parentId).getParentId()).getChildren().add(map.get(parentId));
                deleteKeys.add(parentId);
            }
        }
        for (Long key : deleteKeys) {
            map.remove(key);
        }

        return new ArrayList<>(map.values());
    }

    @Override
    public String savePrivilege(SysPrivilege privilege) {
        QueryWrapper<SysPrivilege> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysPrivilege::getParentId, privilege.getParentId()).eq(SysPrivilege::getName, privilege.getName());
        SysPrivilege sysPrivilege = this.getOne(queryWrapper);

        // 新增
        if (privilege.getId() == null) {
            if (sysPrivilege != null) {
                throw ExceptionResultEnum.ERROR.exception("名称已存在");
            }
            privilege.setId(SystemConstant.getDbUuid());
            privilege.setCreateTime(System.currentTimeMillis());
            this.save(privilege);
        }
        // 修改
        else {
            if (sysPrivilege != null && !Objects.equals(privilege.getId(), sysPrivilege.getId())) {
                throw ExceptionResultEnum.ERROR.exception("名称已存在");
            }
            this.updateById(privilege);
        }
        return String.valueOf(privilege.getId());
    }

    @Override
    public boolean remove(Long id) {
        QueryWrapper<SysRolePrivilege> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysRolePrivilege::getPrivilegeId, id);
        List<SysRolePrivilege> sysRolePrivileges = sysRolePrivilegeService.list(queryWrapper);
        if (sysRolePrivileges != null && sysRolePrivileges.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("权限已绑定，不能删除");
        }
        return this.removeById(id);
    }

    @Override
    public List<String> getRolePrivileges(Long roleId) {
        QueryWrapper<SysRolePrivilege> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysRolePrivilege::getRoleId, roleId);
        List<SysRolePrivilege> sysRolePrivileges = sysRolePrivilegeService.list(queryWrapper);
        List<String> list = null;
        if (sysRolePrivileges != null && sysRolePrivileges.size() > 0) {
            list = sysRolePrivileges.stream().map(m -> String.valueOf(m.getPrivilegeId())).collect(Collectors.toList());
        }
        return list;
    }

    @Override
    public List<PrivilegeDto> listPrivilegeTreeAuth() {
        QueryWrapper<SysPrivilege> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysPrivilege::getProperty, PrivilegePropertyEnum.AUTH.name())
                .or().isNull(SysPrivilege::getProperty)
                .orderByAsc(SysPrivilege::getParentId, SysPrivilege::getSequence);
        List<SysPrivilege> orgList = this.list(queryWrapper);
        Map<Long, PrivilegeDto> map = new LinkedHashMap<>();
        for (SysPrivilege privilege : orgList) {
            PrivilegeDto privilegeDto = new PrivilegeDto();
            privilegeDto.setId(String.valueOf(privilege.getId()));
            privilegeDto.setName(privilege.getName());
            privilegeDto.setUrl(privilege.getUrl());
            privilegeDto.setType(privilege.getType());
            privilegeDto.setParentId(privilege.getParentId());
            privilegeDto.setSequence(privilege.getSequence());
            map.put(privilege.getId(), privilegeDto);
        }

        Iterator<Long> iterator = map.keySet().iterator();
        Set<Long> deleteKeys = new HashSet<>();
        while (iterator.hasNext()) {
            Long parentId = iterator.next();
            if (map.get(parentId).getParentId() != null) {
                map.get(map.get(parentId).getParentId()).getChildren().add(map.get(parentId));
                deleteKeys.add(parentId);
            }
        }
        for (Long key : deleteKeys) {
            map.remove(key);
        }

        return new ArrayList<>(map.values());
    }
}
