package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.entity.SysRole;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.entity.SysUserRole;
import com.qmth.distributed.print.business.enums.RoleTypeEnum;
import com.qmth.distributed.print.business.mapper.SysRoleMapper;
import com.qmth.distributed.print.business.service.CacheService;
import com.qmth.distributed.print.business.service.SysRolePrivilegeService;
import com.qmth.distributed.print.business.service.SysRoleService;
import com.qmth.distributed.print.business.service.SysUserRoleService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 角色表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {

    @Autowired
    private SysUserRoleService sysUserRoleService;

    @Autowired
    private SysRolePrivilegeService sysRolePrivilegeService;

    @Autowired
    private CacheService cacheService;

    @Override
    public IPage<SysRole> list(String name, Boolean enable, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Page<SysRole> page = new Page<>(pageNumber, pageSize);
        IPage<SysRole> roleIPage = this.baseMapper.listPage(page, schoolId, name, enable);
        return roleIPage;
    }

    @Transactional
    @Override
    public boolean saveRole(SysRole role) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        role.setSchoolId(schoolId);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        QueryWrapper<SysRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysRole::getSchoolId, role.getSchoolId()).eq(SysRole::getName, role.getName());
        SysRole sysRole = this.getOne(queryWrapper);
        boolean isSuccess;
        // 新增
        if (Objects.isNull(role.getId())) {
            if (sysRole != null) {
                throw ExceptionResultEnum.ERROR.exception("角色已存在");
            }
            role.setId(SystemConstant.getDbUuid());
            role.setCreateId(sysUser.getId());
            role.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(role);
        }
        // 修改
        else {
            if (sysRole != null && sysRole.getId().longValue() != role.getId().longValue()) {
                throw ExceptionResultEnum.ERROR.exception("角色已存在");
            }

            role.setUpdateId(sysUser.getId());
            role.setUpdateTime(System.currentTimeMillis());

//            if (RoleTypeEnum.CUSTOM.equals(role.getType())) {
                //删除权限
                sysRolePrivilegeService.removeByRoleId(role.getId());
//            }
            isSuccess = this.updateById(role);

        }

//        if (RoleTypeEnum.CUSTOM.equals(role.getType())) {
            // 新增权限
            sysRolePrivilegeService.saveBatch(role);
//        }

        // 更新sys_user_role
        sysUserRoleService.rebindPrivileges(role.getId());

        // 清除缓存
        cacheService.removeUserAuthCache(sysUser.getId());
        cacheService.removeRolePrivilegeCache(role.getId());

        return isSuccess;
    }

    @Override
    public boolean enable(SysRole role) {
        UpdateWrapper<SysRole> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(SysRole::getEnable, role.getEnable()).eq(SysRole::getId, role.getId());
        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean remove(Long id) {
        // 判断绑定用户
        List<SysUserRole> userRoleList = sysUserRoleService.listByRoleId(id);
        if (userRoleList != null && userRoleList.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("用户绑定，不能删除");
        }

        //删除权限
        sysRolePrivilegeService.removeByRoleId(id);
        return this.removeById(id);
    }

    @Override
    public List<Long> getUserRoles(Long userId) {
        QueryWrapper<SysUserRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserRole::getUserId, userId);
        List<SysUserRole> sysUserRoles = sysUserRoleService.list(queryWrapper);
        List<Long> list = null;
        if (sysUserRoles != null && sysUserRoles.size() > 0) {
            list = sysUserRoles.stream().map(m -> m.getRoleId()).collect(Collectors.toList());
        }
        return list;
    }

    @Override
    public List<SysRole> listRolesByUserId(Long id) {
        return this.baseMapper.listRolesByUserId(id);
    }

    @Override
    public List<SysRole> list(Long[] roleIds, String name) {
        QueryWrapper<SysRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysRole::getType, name).in(SysRole::getId, roleIds);
        return this.baseMapper.selectList(queryWrapper);
    }

    @Override
    public List<SysRole> listToUser() {
        Long schoolId = null;
        try {
            Object object = ServletUtil.getRequestHeaderSchoolId();
            schoolId = Long.valueOf(object.toString());
        } catch (Exception e){

        }
        QueryWrapper<SysRole> queryWrapper = new QueryWrapper<>();
        Long finalSchoolId = schoolId;
        queryWrapper.lambda().eq(SysRole::getEnable, true).and(e->e.eq(SysRole::getSchoolId, finalSchoolId).or().isNull(SysRole::getSchoolId));
        return this.list(queryWrapper);
    }
}
