package com.qmth.distributed.print.business.templete.execute;

import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.entity.TBTask;
import com.qmth.distributed.print.business.enums.TaskResultEnum;
import com.qmth.distributed.print.business.enums.TaskStatusEnum;
import com.qmth.distributed.print.business.enums.TaskTypeEnum;
import com.qmth.distributed.print.business.templete.export.AsyncExportTaskTemplete;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.distributed.print.common.contant.SpringContextHolder;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.MessageFormat;
import java.util.Map;
import java.util.StringJoiner;

/**
 * @Description: 异步考务数据导出
 * @Author: CaoZixuan
 * @Date: 2021-04-10
 */
@Service
public class AsyncExaminationExportTemplateService extends AsyncExportTaskTemplete {

    public static final String OBJ_TITLE = "考务数据";
    private final static Logger log = LoggerFactory.getLogger(AsyncExaminationExportTemplateService.class);

    @Override
    public Result exportTask(Map<String, Object> map) {
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);

        StringJoiner stringJoinerSummary = new StringJoiner("\n")
                .add(MessageFormat.format("{0}{1}{2}", FORMAT_TIME, BEGIN_TITLE, OBJ_TITLE));
        tbTask.setStatus(TaskStatusEnum.RUNNING);


        try {
            TaskLogicService taskLogicService = SpringContextHolder.getBean(TaskLogicService.class);
            Map<String, Object> result = taskLogicService.executeExaminationLogic(map);
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", FORMAT_TIME, FINISH_TITLE, result.size(), FINISH_SIZE));
            String path = String.valueOf(result.get(SystemConstant.PATH));
            System.out.println("path = " + path);
            tbTask.setImportFileName(TaskTypeEnum.EXAMINATION_EXPORT.getTitle());
            tbTask.setResultFilePath(path);
            tbTask.setResult(TaskResultEnum.SUCCESS);

        } catch (Exception e) {
            log.error("请求出错", e);
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", FORMAT_TIME, EXCEPTION_TITLE, EXCEPTION_DATA, e.getMessage()));
            tbTask.setResult(TaskResultEnum.ERROR);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {//生成txt文件
            tbTask.setSummary(stringJoinerSummary.toString());
            super.createTxt(tbTask);
        }
        return ResultUtil.ok();
    }
}
