package com.qmth.distributed.print.task.job.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.distributed.print.business.entity.ExamDetail;
import com.qmth.distributed.print.business.entity.ExamPrintPlan;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.service.ExamDetailService;
import com.qmth.distributed.print.business.service.ExamPrintPlanService;
import com.qmth.distributed.print.business.service.OrgCenterDataDisposeService;
import com.qmth.distributed.print.task.job.service.JobService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * @Description: job service impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/31
 */
@Service
public class JobServiceImpl implements JobService {
    private final static Logger log = LoggerFactory.getLogger(JobServiceImpl.class);

    @Resource
    OrgCenterDataDisposeService orgCenterDataDisposeService;

    @Resource
    ExamPrintPlanService examPrintPlanService;

    @Resource
    ExamDetailService examDetailService;

    @Override
    public void updateSchoolInfo() throws IOException {
        orgCenterDataDisposeService.updateSchoolInfo();
    }

    /**
     * 更新印刷计划状态
     */
    @Override
    @Transactional
    public void updatePrintPlan() {
        QueryWrapper<ExamPrintPlan> examPrintPlanQueryWrapper = new QueryWrapper<>();
        examPrintPlanQueryWrapper.lambda().ne(ExamPrintPlan::getStatus, PrintPlanStatusEnum.END);
        List<ExamPrintPlan> examPrintPlanList = examPrintPlanService.list(examPrintPlanQueryWrapper);
        if (Objects.nonNull(examPrintPlanList) && examPrintPlanList.size() > 0) {
            for (ExamPrintPlan examPrintPlan : examPrintPlanList) {
                QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
                examDetailQueryWrapper.lambda().eq(ExamDetail::getPrintPlanId, examPrintPlan.getId());
                List<ExamDetail> examDetailList = examDetailService.list(examDetailQueryWrapper);
                if (Objects.nonNull(examDetailList) && examDetailList.size() > 0) {
                    Long maxEndTime = examDetailList.stream().mapToLong(i -> i.getExamEndTime()).max().getAsLong();
                    if (System.currentTimeMillis() >= maxEndTime.longValue()) {
                        examPrintPlan.setStatus(PrintPlanStatusEnum.END);
                    }
                }
            }
        }
        examPrintPlanService.saveOrUpdateBatch(examPrintPlanList);
    }
}
