package com.qmth.distributed.print.api;


import com.itextpdf.text.log.SysoCounter;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.dto.OrgDto;
import com.qmth.distributed.print.business.entity.SysOrg;
import com.qmth.distributed.print.business.service.SysOrgService;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 学校组织架构 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "组织架构Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.sys}/org")
@Aac(auth = BOOL.FALSE)
public class SysOrgController {

    @Autowired
    private SysOrgService sysOrgService;

    /**
     * 查询机构树
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list() {
        List<OrgDto> orgDtoList = sysOrgService.listOrgTree();
        return ResultUtil.ok(orgDtoList);
    }

    /**
     * 新增/修改
     * @param org
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(@RequestBody SysOrg org) {
        boolean isSuccess = sysOrgService.saveOrg(org);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 启用/禁用
     * @param org
     * @return
     */
    @ApiOperation(value = "启用/禁用")
    @RequestMapping(value = "/enable", method = RequestMethod.POST)
    public Result enable(@RequestBody SysOrg org) {
        boolean isSuccess = sysOrgService.enable(org);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 删除
     * @param org
     * @return
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "/remove", method = RequestMethod.POST)
    public Result remove(@RequestBody SysOrg org) {
        boolean isSuccess = sysOrgService.remove(org.getId());
        return ResultUtil.ok(isSuccess);
    }

}

